/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2025 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */

//---------------------------------------------------------------
// F405VG_UART1
//---------------------------------------------------------------
// Workspace : C:\Project_CubeIDE\Launch_F405VG\L_F405VG_UART1
//---------------------------------------------------------------
// Board : Base-F405VG
// CPU : STM32F405VG

// I/F : IF-F4xxxx-UART-CMOS or IF-F4xxxx-UART-RS232C
//---------------------------------------------------------------

// プログラム開始番地 : 0x08000000
//---------------------------------------------------------------
// UART1 通信試験
//---------------------------------------------------------------
// コマンドを受信して、LEDの点滅間隔を変更します。
// 受信データは、エコーバックします。
//---------------------------------------------------------------
// 受信Bufferのサイズは、
// F405VG_UART1_Config/Dev_Conf.h 内で、
// #define def_Rx1_BufferSize  2048
// と、定義されています。
//---------------------------------------------------------------
// 1回の受信データ数が def_Rx1_BufferSize を越えた場合、
// 越えた分のデータは捨てられます。
//---------------------------------------------------------------
// V001 : 2025/01/29 : えがおのでんし : 初版
//---------------------------------------------------------------



//--------------------------------------------------------------
// 変数の名称について
//--------------------------------------------------------------
// STMicroelectronics社が提供するライブラリに使用している変数は、
// そのまま使用します。
//--------------------------------------------------------------
// えがおのでんし が作成したモジュールの変数名は
// 以下の規則に従います。
//--------------------------------------------------------------
// define宣言の変数は

// #define def_Xxxxx 100

// のように名前の先頭に def_ を付けています。
//--------------------------------------------------------------
// 構造体の定義

// struct sXxx{
// };

// のように、名前を指定しています。
//--------------------------------------------------------------
// 構造体の宣言

// struct sXxx structXxx;

// のように宣言しています。
//--------------------------------------------------------------
// 変数はbit幅と signed/unsigned が分かるように
// それぞれ次のように名前を付けています。

// int8_t int8_xxxxx;
// uint8_t uint8_xxxxx;

// int16_t int16_xxxxx;
// uint16_t uint16_xxxxx;

// int32_t int32_xxxxx;
// uint32_t uint32_xxxxx;
//--------------------------------------------------------------
// ポインタの場合はそれぞれ先頭に p を付けています。

// int8_t *pint8_xxxxx;
// uint8_t *puint8_xxxxx;

// int16_t *pint16_xxxxx;
// uint16_t *puint16_xxxxx;

// int32_t *pint32_xxxxx;
// uint32_t *puint32_xxxxx;

// struct sXxx *pstructXxx
//--------------------------------------------------------------
// グローバル変数には先頭に GLB_ を付けています。

// int16_t GLB_int16_xxxxx;

// struct sXxx GLB_structXxx;
//--------------------------------------------------------------
// volatile : 変数名の前に v を付けています。

// volatile int16_t GLB_int16_vXxxxx;
//--------------------------------------------------------------
// static変数には先頭に STC_ を付けています。

// static int16_t STC_int16_xxxxx;
//--------------------------------------------------------------
// 配列には先頭に a を付けています。

// int16_t int16_aXxxxx[n];
// int16_t GLB_int16_aXxxxx[n];
//--------------------------------------------------------------

/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

#include "Blink_LED_ST.h"
#include "H_STM32F4_UART_1.h"
#include "Communication_Test_UARTx.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{

  /* USER CODE BEGIN 1 */

  //-----------------------------------------------------------------
  // main関数内では、以下の処理番号の変数、
  // STC_uint8_JobNum_LED と
  // STC_uint8_JobNum_UART は

  // while(1){
  // }

  // の永久ループ内で使用しているので、
  // static宣言をする必要はない。

  // しかし、main関数以外の全ての関数内では
  // 処理番号をstatic宣言をする必要があるので、
  // 合わせるために、ここでも static宣言をしている。
  //-----------------------------------------------------------------
  static uint8_t STC_uint8_JobNum_LED;   // LED点滅処理番号
  static uint8_t STC_uint8_JobNum_UART;  // UART通信処理番号


  //------------------------------------------------------------
  // 呼び出す関数を case 0: から実行させるために
  // 処理番号に 0 をセットします。
  //------------------------------------------------------------
  STC_uint8_JobNum_LED = 0;   // LED点滅処理番号初期化
  STC_uint8_JobNum_UART = 0;  // UART通信処理番号初期化

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
    //------------------------------------------------------
    // ST LED点滅
    //------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Blink_LED_ST *pstructParam_Blink_LED_ST :
    //              ST LED点滅用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG
    //------------------------------------------------------
    Blink_LED_ST(&STC_uint8_JobNum_LED,
                 &GLB_structParam_Blink_LED_ST);



    //------------------------------------------------------------
    // UARTx通信試験
    //------------------------------------------------------------
    // 処理内容:

    //  1. UARTの初期化がされていない場合、初期化します。
    //  2. 受信が発生するのを待ち、受信データを取得します。
    //  3. 受信データを判定して、内容に従い処理を行います。
    //  4. 受信データによる処理の結果、
    //          送信データがセットされた場合、送信処理を行います。
    //------------------------------------------------------------
    // 引数;
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // USART_TypeDef* USARTx :
    //    where x can be 1, 2, 3, 4, 5 or 6
    //    to select the USART or UART peripheral.

    // UART_HandleTypeDef *pHandle_UARTx :
    //  pointer to a UART_HandleTypeDef structure that contains
    //  the configuration information
    //                      for the specified UART module.

    // uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
    // uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

    //  uint8_t *puint8_aReceiveData_UART :
    //              UART通信処理用 受信データBufferのポインタ

    // volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
    // volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

    // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
    //              Handle UART用パラメータの構造体のポインタ


    // 戻り値;
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //------------------------------------------------------------
    Communication_Test_UARTx(&STC_uint8_JobNum_UART,
                             USART1,
                             &Handle_UART1,
                             GLB_uint8_aTx1_Buffer,
                             GLB_uint8_aRx1_Buffer,
                             GLB_uint8_aReceiveData_UART1,
                             (volatile uint16_t)def_Tx1_BufferSize,
                             (volatile uint16_t)def_Rx1_BufferSize,
                             &GLB_structParam_Handle_UART1);
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 8;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
/* USER CODE BEGIN MX_GPIO_Init_1 */
/* USER CODE END MX_GPIO_Init_1 */

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOH_CLK_ENABLE();

/* USER CODE BEGIN MX_GPIO_Init_2 */
/* USER CODE END MX_GPIO_Init_2 */
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
