/*
 * H_Common_STM32F4_UART.c
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#include "H_Common_STM32F4_UART.h"

#include "Time_Interval.h"



//----------------------------------------------------------
// Handle UARTx用パラメータの構造体を初期化
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
// uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

//  uint8_t *puint8_aReceiveData_UART :
//              UART通信処理用 受信データBufferのポインタ

// volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
// volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値;
//    なし
//----------------------------------------------------------
void Initialize_Parameter_UARTx(uint8_t *puint8_aTx_Buffer,
                        uint8_t *puint8_aRx_Buffer,
                        uint8_t *puint8_aReceiveData_UART,
                        volatile uint16_t uint16_vTx_Transfer,
                        volatile uint16_t uint16_vRx_Receive,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
  //--------------------------------------------------------
  // Bufferのポインタとサイズをセット
  //--------------------------------------------------------
  // 送信Bufferのポインタをセット
  pstructParam_Handle_UARTx->puint8_aTx_Buffer = puint8_aTx_Buffer;
  // 受信Bufferのポインタをセット
  pstructParam_Handle_UARTx->puint8_aRx_Buffer = puint8_aRx_Buffer;

  // UART通信処理用 受信データBufferのポインタをセット
  pstructParam_Handle_UARTx->puint8_aReceiveData_UART
                                    = puint8_aReceiveData_UART;

  // 送信Bufferのサイズをセット
  pstructParam_Handle_UARTx->uint16_vTx_Transfer = uint16_vTx_Transfer;
  // 受信Bufferのサイズをセット
  pstructParam_Handle_UARTx->uint16_vRx_Receive = uint16_vRx_Receive;
  //--------------------------------------------------------


  //--------------------------------------------------------
  // Handle UART用パラメータに初期値をセット
  //--------------------------------------------------------
  pstructParam_Handle_UARTx->uint32_BaudRate_UART = 115200;
  pstructParam_Handle_UARTx->uint32_Parity_UART = UART_PARITY_NONE;


  pstructParam_Handle_UARTx->uint16_vTx_Counter = 0;
  pstructParam_Handle_UARTx->uint16_vRx_Counter = 0;

  pstructParam_Handle_UARTx->uint16_vRx_ReadPointer = 0;
  pstructParam_Handle_UARTx->uint16_vRx_WritePointer = 0;
  pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave = 0;

  // 受信Delimiter : 検出なし
  pstructParam_Handle_UARTx->uint8_vRx_Delimiter = (uint8_t)0;


  pstructParam_Handle_UARTx->int16_vUART_ReceiveTimeOutVal = -1;
                                    // 受信TimeOut値 : Timeoutなし
  pstructParam_Handle_UARTx->int16_vUART_ReceiveTimeOut = -1;
                                    // 受信TimeOut Counter
  pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTimeVal = -1;
                                    // 受信終了Timer値
  pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime = -1;
                                    // 受信終了Timer Counter


  pstructParam_Handle_UARTx->int16_vUART_Error = 0;


  pstructParam_Handle_UARTx->int16_vUART_InitDone = 0;
                                    // 初期化済みフラグクリア
  //--------------------------------------------------------
}



//----------------------------------------------------------
// UARTx初期化
//----------------------------------------------------------
// 引数;
// USART_TypeDef* USARTx :
//      where x can be 1, 2, 3, 4, 5 or 6
//      to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint32_t uint32_BaudRate : BaudRate
// uint32_t uint32_Parity : Parity
// uint8_t *puint8_aBuffer_UARTx : 受信Bufferのポインタ

// volatile int16_t *pint16_vUARTx_InitDone :
//                          初期化済みフラグ変数のポインタ

// 戻り値;
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t Initialize_UARTx(USART_TypeDef* USARTx,
                          UART_HandleTypeDef *pHandle_UARTx,
                          uint32_t uint32_BaudRate,
                          uint32_t uint32_Parity,
                          uint8_t *puint8_aBuffer_UARTx,
                    volatile int16_t *pint16_vUARTx_InitDone)
{
  HAL_StatusTypeDef hal_Status;


  // Configure the UART peripheral
  // Put the USART peripheral in the Asynchronous mode
  //                                        (UART Mode)
  pHandle_UARTx->Instance = USARTx;

  pHandle_UARTx->Init.BaudRate = uint32_BaudRate;

  if(uint32_Parity == UART_PARITY_NONE){
    pHandle_UARTx->Init.WordLength   = UART_WORDLENGTH_8B;
  }
  else{
    pHandle_UARTx->Init.WordLength   = UART_WORDLENGTH_9B;
  }

  pHandle_UARTx->Init.StopBits     = UART_STOPBITS_1;
  pHandle_UARTx->Init.Parity       = uint32_Parity;
  pHandle_UARTx->Init.HwFlowCtl    = UART_HWCONTROL_NONE;
  pHandle_UARTx->Init.Mode         = UART_MODE_TX_RX;
  pHandle_UARTx->Init.OverSampling = UART_OVERSAMPLING_16;

  hal_Status = HAL_UART_Init(pHandle_UARTx);
  if(hal_Status != HAL_OK){
    return(1);  // NG
  }


  // Interrupt
  hal_Status = HAL_UART_Receive_IT(pHandle_UARTx,
                                   puint8_aBuffer_UARTx, 1);
  if(hal_Status != HAL_OK){
    return(1);  // NG
  }


  // 初期化フラグON
  *pint16_vUARTx_InitDone = 1;


  return(0);  // OK
}



//----------------------------------------------------------
// UARTx 送信処理 : 送信Bufferに送信データをセットして、
//          送信データ数をセットした状態でCallしてください。
//----------------------------------------------------------
// 送信Buffer : pstructParam_Handle_UARTx->puint8_aTx_Buffer
// 送信データ数 : pstructParam_Handle_UARTx->uint16_vTx_Transfer
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値;
//   -1 : 処理中
//    0 : 送信終了
//    1 : エラー
//----------------------------------------------------------
int16_t Send_UARTx(uint8_t *puint8_JobNum,
                   UART_HandleTypeDef *pHandle_UARTx,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_Time;  // Time_Interval処理番号

  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言は、
  // 使用する関数内で、必ず static で宣言してください。
  //------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言
  //------------------------------------------------------
  static struct sParam_Time_Interval STC_structParam_Time_Interval;


  int16_t int16_Return;

  HAL_StatusTypeDef hal_Status;


  switch(*puint8_JobNum)
  {
  case 0:
    pstructParam_Handle_UARTx->uint16_vTx_Counter = 0;
                                    // 送信カウンタ初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    // Start the transmission process
    hal_Status = HAL_UART_Transmit_IT(
                    pHandle_UARTx,
                    pstructParam_Handle_UARTx->puint8_aTx_Buffer,
                    pstructParam_Handle_UARTx->uint16_vTx_Transfer);
    //------------------------------------------
    // 送信開始処理が成功したか判定する。
    //------------------------------------------
    if(hal_Status != HAL_OK){
      // 送信開始処理失敗
      *puint8_JobNum = 0; // 処理番号初期化
      int16_Return = 1;   // NG終了
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化
    // 送信タイムアウト値をセット
    STC_structParam_Time_Interval.uint32_Interval
        = (uint32_t)pstructParam_Handle_UARTx->uint16_vTx_Transfer * 4;


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------
    // 送信タイムアウト判定
    //----------------------------------
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数 :
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値 :
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == 0){
      // Timeout
      *puint8_JobNum = 0;  // 処理番号初期化
      int16_Return = 1;    // NG終了
      break;
    }


    //--------------------------------------
    // UART1 送信終了待ち
    //--------------------------------------
    if(pstructParam_Handle_UARTx->uint16_vTx_Counter
        < pstructParam_Handle_UARTx->uint16_vTx_Transfer){
      // 送信中
      int16_Return = -1;  //処理継続
      break;  //  処理番号維持 : 現在の処理番号をループ
    }


    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 0;    // OK終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// UARTx 受信処理
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// uint8_t *puint8_aReceiveBuffer :
//                      受信データを格納するBufferのポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値;
//    -2 : エラー発生
//    -1 : 受信処理中
//     0 : 受信なし
// 1以上 : 受信byte数
//----------------------------------------------------------
int16_t Receive_UARTx(uint8_t *puint8_JobNum,
                      uint8_t *puint8_aReceiveBuffer,
        struct sParam_Handle_UARTx *pstructParam_Handle_UARTx)
{
  uint16_t uint16_Length;

  int16_t int16_Return;


  switch(*puint8_JobNum){
  case 0:
    // 初期化処理なし


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------
    // エラー発生を監視
    // エラーが発生した場合、
    //  pstructParam_Handle_UARTx->int16_vUART_InitDone = 0 になる。
    //--------------------------------------
    if(pstructParam_Handle_UARTx->int16_vUART_InitDone == 0){
      //------------------------------------
      // エラー発生時の処理
      //------------------------------------
      //--------------------------------------------------------
      *puint8_JobNum = 0;  // 処理番号初期化
      int16_Return = -2;   // エラー発生
      break;  // エラー終了
      //--------------------------------------------------------
    }


    //----------------------------------------------------------
    // 受信待ち
    //----------------------------------------------------------
    if(pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime != 0){
      // 受信処理中
      int16_Return = -1;  //処理継続
      break;  // 処理番号維持 : 現在の処理番号をループ
    }
    //------------------------------------------------------
    // 受信終了まで、この下にいかない。
    //------------------------------------------------------


    // 受信終了Timer初期化
    pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime = -1;

    pstructParam_Handle_UARTx->uint16_vRx_Counter = 0;
                                // 受信データ数カウンタクリア


    //----------------------------------------------------------
    // 受信データを抜き出す。
    //----------------------------------------------------------
    // 受信Bufferの読み込み開始位置と書き込み開始位置を使用して
    // 受信データを抜き出した後、読み込み開始位置を更新する。
    //----------------------------------------------------------
    // 引数;
    // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
    //              Handle UART用パラメータの構造体のポインタ

    // uint8_t *puint8_aExtractData :
    //              抜き出したデータを格納するBufferのポインタ

    // 戻り値;
    //     0 : データなし
    // 1以上 : 受信データ数
    //----------------------------------------------------------
    uint16_Length = Extract_Receive_Data(pstructParam_Handle_UARTx,
                                         puint8_aReceiveBuffer);


    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = (int16_t)uint16_Length;  // 受信byte数をセット
    break;  // 受信終了

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 0;    // 受信なし
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// 受信データを抜き出す。
//----------------------------------------------------------
// 受信Bufferの読み込み開始位置と書き込み開始位置を使用して
// 受信データを抜き出した後、読み込み開始位置を更新する。
//----------------------------------------------------------
// 引数;
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ

// uint8_t *puint8_aExtractData :
//              抜き出したデータを格納するBufferのポインタ

// 戻り値;
//     0 : データなし
// 1以上 : 受信データ数
//----------------------------------------------------------
uint16_t Extract_Receive_Data(
            struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx,
                              uint8_t *puint8_aExtractData)
{
  uint16_t uint16_Length;
  uint16_t uint16_I;


  // 変数チェック
  if(pstructParam_Handle_UARTx->uint16_vRx_ReadPointer
                    >= pstructParam_Handle_UARTx->uint16_vRx_Receive){
    return(0);  // データなし
  }

  if(pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave
                    >= pstructParam_Handle_UARTx->uint16_vRx_Receive){
    return(0);  // データなし
  }


  //------------------------------------------------------------
  // 受信データ数算出
  //------------------------------------------------------------
  if(pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave
      == pstructParam_Handle_UARTx->uint16_vRx_ReadPointer){
    uint16_Length = pstructParam_Handle_UARTx->uint16_vRx_Receive;
  }
  else if(pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave
      > pstructParam_Handle_UARTx->uint16_vRx_ReadPointer){
    uint16_Length
        = pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave
        - pstructParam_Handle_UARTx->uint16_vRx_ReadPointer;
  }
  else{
    uint16_Length
        = pstructParam_Handle_UARTx->uint16_vRx_Receive
        - pstructParam_Handle_UARTx->uint16_vRx_ReadPointer
        + pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave;
  }


  if(uint16_Length > 0){
    // 受信データを格納Bufferにセット
    for(uint16_I = 0; uint16_I < uint16_Length; uint16_I++){
      puint8_aExtractData[uint16_I]
        = pstructParam_Handle_UARTx->puint8_aRx_Buffer[
            pstructParam_Handle_UARTx->uint16_vRx_ReadPointer];
      pstructParam_Handle_UARTx->uint16_vRx_ReadPointer++;
      if(pstructParam_Handle_UARTx->uint16_vRx_ReadPointer
                    == pstructParam_Handle_UARTx->uint16_vRx_Receive){
        pstructParam_Handle_UARTx->uint16_vRx_ReadPointer = 0;
      }
    }
  }

  return(uint16_Length);
}
